@@ -5,112 +5,27 @@ from pathlib import Path
 import pandas
 import re
 import numpy as np
-from damask import DADF5
+from ruamel.yaml import YAML
 
 from damask_parse.utils import (
     get_header_lines,
     get_num_header_lines,
-    get_HDF5_incremental_quantity
+    get_HDF5_incremental_quantity,
+    validate_volume_element,
+    validate_element_material_idx,
 )
+from damask_parse.legacy.readers import parse_microstructure, parse_texture_gauss
 
 __all__ = [
-    'read_table',
     'read_geom',
     'read_spectral_stdout',
     'read_spectral_stderr',
     'read_HDF5_file',
+    'read_material',
+    'geom_to_volume_element',
 ]
 
 
-def parse_microstructure(ms_str):
-    """Parse a DAMASK microstructure definition from within a string.
-
-    Parameters
-    ----------
-    ms_str : str
-        String that contains a microstructure part.
-
-    Returns
-    -------
-    microstructure : dict
-        Dict representing the parsed microstructure; with the following keys:
-            phase_idx : ndarray of int
-                Zero-indexed integer index array of phases.
-            texture_idx : ndarray of int
-                Zero-indexed integer index array of orientations.
-            fraction : ndarray of float                
-    """
-
-    pat = (r'\[[G|g]rain(\d+)\][\s\S]crystallite\s(\d+)[\s\S]\(constituent\)\s+phase\s+'
-           r'(\d+)\s+texture\s+(\d+)\s+fraction\s+(\d\.\d+)')
-    matches = re.findall(pat, ms_str)
-    if not matches:
-        raise ValueError('No DAMASK microstructure part found in the string.')
-
-    phase_idx = []
-    texture_idx = []
-    fraction = []
-    for i in matches:
-        phase_idx.append(int(i[2]))
-        texture_idx.append(int(i[3]))
-        fraction.append(float(i[4]))
-
-    microstructure = {
-        'phase_idx': np.array(phase_idx) - 1,
-        'texture_idx': np.array(texture_idx) - 1,
-        'fraction': np.array(fraction),
-    }
-
-    return microstructure
-
-
-def parse_texture_gauss(texture_str):
-    """Parse a DAMASK Gauss texture definition from within a string.
-
-    Parameters
-    ----------
-    texture_str : str
-        String that contains a "Gauss" texture part (i.e. euler angles for each grain).
-
-    Returns
-    -------
-    texture : dict
-        Dict representing the parsed texture; with the following keys:
-            euler_angles : ndarray of shape (N, 3)
-                Array of row vectors of Euler angles.
-            euler_angle_labels : list of str
-                Labels of the Euler angles.
-            fraction : ndarray or NoneType
-            scatter : ndarray or NoneType
-    """
-
-    pat = (r'\[[G|g]rain(\d+)\][\s\S]\(gauss\)\s+phi1\s+(\d+\.?\d+)\s+Phi\s+(\d+\.?\d+)'
-           r'\s+phi2\s+(\d+\.?\d+)(?:\s+scatter\s+(\d+\.?\d+)\s+fraction\s+(\d+\.?\d+))?'
-           r'(?:\s+)?')
-    matches = re.findall(pat, texture_str)
-    if not matches:
-        raise ValueError('No DAMASK texture part found in the string.')
-
-    eulers = []
-    fraction = []
-    scatter = []
-    for texture_section in matches:
-        eulers.append([float(texture_section[i]) for i in [1, 2, 3]])
-        if texture_section[4]:
-            fraction.append(float(texture_section[4]))
-        if texture_section[5]:
-            scatter.append(float(texture_section[5]))
-
-    texture = {
-        'euler_angles': np.array(eulers),
-        'euler_angle_labels': ['phi1', 'Phi', 'phi2'],
-        'fraction': np.array(fraction) if fraction else None,
-        'scatter': np.array(scatter) if scatter else None,
-    }
-
-    return texture
-
-
 def parse_increment_iteration(inc_iter_str):
 
     float_pat = r'-?\d+\.\d+'
@@ -227,129 +142,6 @@ def parse_increment(inc_str):
     return parsed_inc
 
 
-def read_table(path, use_dataframe=False, combine_array_columns=True,
-               ignore_duplicate_cols=False, check_header=True):
-    """Read the data from a DAMASK-generated ASCII table file, as generated by
-    the DAMASK post-processing command named `postResults`.
-
-    Parameters
-    ----------
-    path : str or Path
-        Path to the DAMASK table file.
-    use_dataframe : bool, optional
-        If True, a Pandas DataFrame is returned. Otherwise, a dict of Numpy
-        arrays is returned. By default, set to False.        
-    combine_array_columns : bool, optional
-        If True, columns that represent elements of an array (e.g. stress) are
-        combined into a single column. By default, set to True.
-    ignore_duplicate_cols : bool, optional
-        If True, duplicate columns (as detected by the `mangle_dupe_cols`
-        option of `pandas_read_csv` function) are ignored. Otherwise, an
-        exception is raised. By default, set to False.
-    check_header : bool, optional
-        Check that the command `postResults` appears in the header, i.e. check
-        that the file is indeed likely to be a DAMASK table file. By default,
-        set to True.
-
-    Returns
-    -------
-    outputs : dict
-        The data in the table file, represented either as a Pandas DataFrame
-        (if `use_dataframe` is True) or a dict of Numpy arrays (if
-        `use_dataframe` if False).
-
-    TODO: parse all "array-like" columns in one-dimensional Numpy arrays, and
-    then reshape known columns into correct shapes.
-
-    """
-
-    arr_shape_lookup = {
-        12: [4, 3],
-        9: [3, 3],
-        3: [3],
-        4: [4],
-    }
-
-    header = get_header_lines(path)
-    header_num = len(header)
-
-    if check_header:
-        if 'postResults' not in header[0]:
-            msg = (
-                '"postResults" does not appear in the header of the supposed '
-                'table file. If you want to ignore this fact, call the '
-                '`read_table` function with the parameter '
-                '`check_header=False`.'
-            )
-            raise ValueError(msg)
-
-    df = pandas.read_csv(str(path), delim_whitespace=True, header=header_num)
-
-    if not ignore_duplicate_cols:
-        if np.any(df.columns.str.replace(r'(\.\d+)$', '').duplicated()):
-            msg = (
-                'It appears there are duplicated columns in the table. If you '
-                'want to ignore this fact, call the `read_table` function with'
-                ' the parameter `ignore_duplicate_cols=True`.'
-            )
-            raise ValueError(msg)
-
-    arr_sizes = None
-    if combine_array_columns or not use_dataframe:
-        # Find number of elements for each "array" column:
-        arr_sizes = {}
-        for header in df.columns.values:
-            match = re.match(r'([0-9]+)_(.+)', header)
-            if match:
-                arr_name = match.group(2)
-                if arr_name in arr_sizes:
-                    arr_sizes[arr_name] += 1
-                else:
-                    arr_sizes.update({
-                        arr_name: 1
-                    })
-
-        # Check for as yet "unsupported" array dimensions:
-        bad_num_elems = set(arr_sizes.values()) - set(arr_shape_lookup.keys())
-        if len(bad_num_elems) > 0:
-            msg = (
-                '"Array" columns must have one of the following number of '
-                'elements: {}. However, there are columns with the following '
-                'numbers of elements: {}'.format(
-                    list(arr_shape_lookup.keys()), list(bad_num_elems)
-                )
-            )
-            raise ValueError(msg)
-
-    if combine_array_columns:
-        # Add arrays as single columns:
-        for arr_name, arr_size in arr_sizes.items():
-            arr_idx = ['{}_{}'.format(i, arr_name)
-                       for i in range(1, arr_size + 1)]
-            df[arr_name] = df[arr_idx].values.tolist()
-            # Remove individual array columns:
-            df = df.drop(arr_idx, axis=1)
-
-    outputs = df
-
-    if not use_dataframe:
-        # Transform each column into a Numpy array:
-        arrays = {}
-        for header in df.columns.values:
-            val = np.array(df[header])
-
-            if header in arr_sizes:
-                shp = tuple([-1] + arr_shape_lookup[arr_sizes[header]])
-                val = np.array([*df[header]]).reshape(shp)
-
-            arrays.update({
-                header: val
-            })
-            outputs = arrays
-
-    return outputs
-
-
 def read_geom(geom_path):
     """Parse a DAMASK geometry file into a volume element.
 
@@ -360,31 +152,29 @@ def read_geom(geom_path):
 
     Returns
     -------
-    volume_element : dict
-        Dictionary that represents the volume element parsed from the geometry
-        file, with keys:
-            voxel_grain_idx : 3D ndarray of int
+    geometry : dict
+        Dictionary of the parsed data from the geometry file, with keys:
+            element_material_idx : ndarray of shape equal to `grid_size` of int
                 A mapping that determines the grain index for each voxel.
-            voxel_homogenization_idx : 3D ndarray of int
-                A mapping that determines the homogenization scheme (via an integer index)
-                for each voxel.
-            size : list of length three, optional
+            grid_size : ndarray of int of size 3
+                Resolution of volume element discretisation in each direction.
+            size : list of length 3
                 Volume element size. By default set to unit size: [1, 1, 1].
-            origin : list of length three, optional
-                Volume element origin. By default: [0, 0, 0].
-            grid : ndarray of int of size 3
-                Resolution of volume element discretisation in each direction. This will
-                equivalent to the shape of `voxel_grain_idx`.
+            origin : list of length 3
+                Volume element origin. By default: [0, 0, 0].                
+            material_homog_idx : 1D ndarray of str
+                Determines the homogenization scheme for each material.
             orientations : dict
                 Dict containing the following keys:
-                    euler_angles : ndarray of shape (N, 3)
-                        Array of N row vectors of Euler angles.
-                    euler_angle_labels : list of str
-                        Labels of the Euler angles.
-            grain_phase_label_idx : 1D ndarray of int
-                Zero-indexed integer index array mapping a grain to its phase.
-            grain_orientation_idx : 1D ndarray of int
-                Zero-indexed integer index array mapping a grain to its orientation.
+                    euler_angles : ndarray of shape (R, 3) of float
+                        Array of R row three-vectors of Euler angles. Specified as proper
+                        Euler angles in the Bunge convention. (Rotations are about Z,
+                        new X, new new Z.)
+            constituent_phase_label_idx : 1D ndarray of int
+                Zero-indexed integer index array mapping a constituent to its phase index.
+            constituent_orientation_idx : 1D ndarray of int
+                Zero-indexed integer index array mapping a constituent to its orientation
+                index.
             meta : dict
                 Any meta information associated with the generation of this volume
                 element.
@@ -397,32 +187,34 @@ def read_geom(geom_path):
 
         lines = handle.read()
 
-        grid = None
+        grid_size = None
         grid_pat = r'grid\s+a\s+(\d+)\s+b\s+(\d+)\s+c\s+(\d+)'
         grid_match = re.search(grid_pat, lines)
         if grid_match:
-            grid = [int(i) for i in grid_match.groups()]
+            grid_size = [int(i) for i in grid_match.groups()]
         else:
             raise ValueError('`grid` not specified in file.')
 
-        grain_idx_2d = np.zeros((grid[1] * grid[2], grid[0]), dtype=int)
+        element_material_idx = []
         for ln_idx, ln in enumerate(lines.splitlines()):
             ln_split = ln.strip().split()
             if ln_idx > num_header:
-                arr_idx = ln_idx - (num_header + 1)
-                grain_idx_2d[arr_idx] = [int(i) for i in ln_split]
-        voxel_grain_idx = grain_idx_2d.reshape(grid[::-1]).swapaxes(0, 2)
-        voxel_grain_idx -= 1  # zero-indexed
+                element_material_idx.extend([int(i) for i in ln_split])
 
-        grain_phase_label_idx = None
-        grain_orientation_idx = None
+        element_material_idx = np.array(element_material_idx).reshape(grid_size[::-1])
+        element_material_idx = element_material_idx.swapaxes(0, 2)
+        element_material_idx -= 1  # zero-indexed
+        num_mats = validate_element_material_idx(element_material_idx)
+
+        constituent_phase_label_idx = None
+        constituent_orientation_idx = None
         pat = r'\<microstructure\>[\s\S]*\(constituent\).*'
         ms_match = re.search(pat, lines)
         if ms_match:
             ms_str = ms_match.group()
             microstructure = parse_microstructure(ms_str)
-            grain_phase_label_idx = microstructure['phase_idx']
-            grain_orientation_idx = microstructure['texture_idx']
+            constituent_phase_label_idx = microstructure['phase_idx']
+            constituent_orientation_idx = microstructure['texture_idx']
 
         orientations = None
         pat = r'\<texture\>[\s\S]*\(gauss\).*'
@@ -431,12 +223,12 @@ def read_geom(geom_path):
             texture_str = texture_match.group()
             orientations = parse_texture_gauss(texture_str)
 
-        # Check indices in `grain_orientation_idx` are valid, given `orientations`:
+        # Check indices in `constituent_orientation_idx` are valid, given `orientations`:
         if (
-            np.min(grain_orientation_idx) < 0 or
-            np.max(grain_orientation_idx) > len(orientations['euler_angles'])
+            np.min(constituent_orientation_idx) < 0 or
+            np.max(constituent_orientation_idx) > len(orientations['euler_angles'])
         ):
-            msg = 'Orientation indices in `grain_orientation_idx` are invalid.'
+            msg = 'Orientation indices in `constituent_orientation_idx` are invalid.'
             raise ValueError(msg)
 
         # Parse header information:
@@ -454,30 +246,31 @@ def read_geom(geom_path):
 
         homo_pat = r'homogenization\s+(\d+)'
         homo_match = re.search(homo_pat, lines)
-        vox_homo = None
+        material_homog_idx = None
         if homo_match:
-            vox_homo = np.zeros_like(voxel_grain_idx).astype(int)
-            vox_homo[:] = int(homo_match.group(1)) - 1  # zero-indexed
+            # Same homogenization for each material ID:
+            homog_idx = int(homo_match.group(1)) - 1  # zero-indexed
+            material_homog_idx = np.zeros(num_mats).astype(int) + homog_idx
 
         com_pat = r'(geom_.*)'
         commands = re.findall(com_pat, lines)
 
-        volume_element = {
-            'grid': grid,
+        geometry = {
+            'grid_size': grid_size,
             'size': size,
             'origin': origin,
             'orientations': orientations,
-            'voxel_grain_idx': voxel_grain_idx,
-            'voxel_homogenization_idx': vox_homo,
-            'grain_phase_label_idx': grain_phase_label_idx,
-            'grain_orientation_idx': grain_orientation_idx,
+            'element_material_idx': element_material_idx,
+            'material_homog_idx': material_homog_idx,
+            'constituent_phase_label_idx': constituent_phase_label_idx,
+            'constituent_orientation_idx': constituent_orientation_idx,
             'meta': {
                 'num_header': num_header,
                 'commands': commands,
             },
         }
 
-    return volume_element
+    return geometry
 
 
 def read_spectral_stdout(path):
@@ -581,7 +374,7 @@ def read_HDF5_file(hdf5_path, incremental_data, operations=None):
                 The name by which the quantity will be stored in the output dict.
             path: str
                 The HDF5 "path" to the dataset.
-            transform: list of dict, optional
+            transforms: list of dict, optional
                 If specified this is a list of dicts, each with the following keys:
                     sum_along_axes : int, optional
                         If specified, take the sum the array along this axis.
@@ -604,13 +397,13 @@ def read_HDF5_file(hdf5_path, incremental_data, operations=None):
         Dict with keys determined by the `incremental_data` list.
 
     """
-
-    sim_data = DADF5(hdf5_path)
+    from damask import Result
+    sim_data = Result(hdf5_path)
 
     for op in operations or []:
         func = getattr(sim_data, op['name'], None)
         if not func:
-            raise AttributeError(f'The DADF5 object has no attribute: {op["name"]}.')
+            raise AttributeError(f'The Result object has no attribute: {op["name"]}.')
         else:
             func(**op['args'])
 
@@ -654,3 +447,132 @@ def read_HDF5_file(hdf5_path, incremental_data, operations=None):
         })
 
     return volume_element_response
+
+
+def read_material(path):
+    """Parse a DAMASK material.yaml input file.
+
+    Parameters
+    ----------
+    path : str or Path
+        Path to the DAMASK material.yaml file.    
+
+    Returns
+    -------
+    material_data : dict
+        Parsed data from the DAMASK material.yaml file. Keys are:
+            phases : dict
+                The "phase" dict contained within the material file.
+            homog_schemes : dict
+                The "homogenization" dict contained within the material file.
+            volume_element : dict            
+                Dict representing the volume element. The distribution of materials
+                across the elements (i.e. keys `element_material_idx` and `grid_size`)
+                are not included, since this information is not contained in the
+                material file. With keys:
+                    constituent_material_idx : ndarray of shape (N,) of int
+                        Determines the material to which each constituent belongs, where N
+                        is the number of constituents.
+                    constituent_material_fraction: ndarray of shape (N,) of float
+                        The fraction that each constituent occupies within its respective
+                        material, where N is the number of constituents.
+                    constituent_phase_label : ndarray of shape (N,) of str
+                        Determines the phase label of each constituent, where N is the
+                        number of constituents.
+                    constituent_orientation_idx : ndarray of shape (N,) of int
+                        Determines the orientation (as an index into `orientations`)
+                        associated with each constituent, where N is the number of
+                        constituents.
+                    material_homog : ndarray of shape (M,) of str
+                        Determines the homogenization scheme (from a list of available
+                        homogenization schemes defined elsewhere) to which each material
+                        belongs, where M is the number of materials.
+                    orientations : dict, optional
+                        Dict containing the following keys:
+                            type : str
+                                Value is "quat".
+                            quaternions : ndarray of shape (R, 4) of float, optional
+                                Array of R row four-vectors of unit quaternions. Specify
+                                either `quaternions` or `euler_angles`.        
+
+    """
+
+    yaml = YAML(typ='safe')
+    material_dat = yaml.load(Path(path))
+
+    material_homog = []
+    const_material_idx = []
+    const_material_fraction = []
+    const_phase_label = []
+    const_orientation_idx = []
+    orientations = {
+        'type': 'quat',
+        'quaternions': [],
+    }
+
+    for mat_idx, material in enumerate(material_dat['microstructure']):
+        material_homog.append(material['homogenization'])
+        for const in material['constituents']:
+            const_material_idx.append(mat_idx)
+            const_material_fraction.append(const['fraction'])
+            const_phase_label.append(const['phase'])
+            orientations['quaternions'].append(const['orientation'])
+            const_orientation_idx.append(len(const_orientation_idx))
+
+    vol_elem = {
+        'constituent_material_idx': const_material_idx,
+        'constituent_material_fraction': const_material_fraction,
+        'constituent_phase_label': const_phase_label,
+        'constituent_orientation_idx': const_orientation_idx,
+        'material_homog': material_homog,
+        'orientations': orientations,
+    }
+    material_data = {
+        'volume_element': vol_elem,
+        'phases': material_dat['phase'],
+        'homog_schemes': material_dat['homogenization'],
+    }
+    material_data['volume_element'] = validate_volume_element(**material_data)
+
+    return material_data
+
+
+def geom_to_volume_element(geom_path, phase_labels, homog_label):
+    """Read a DAMASK geom file and parse to a volume element.
+
+    Parameters
+    ----------
+    geom_path : str or Path
+        Path to the DAMASK geometry file. The geom file must include texture and
+        microstructure parts in its header.
+    phase_labels : list or ndarray of str, optional
+        List of phase labels to associate with the constituents. The first list element is
+        the phase label that will be associated with all of the geometrical elements
+        for which an orientation is also specified. Additional list elements are
+        phase labels for geometrical elements for which no orientations are
+        specified. For instance, if the DAMASK command `geom_canvas` was used on
+        a geom file to generate a new geom file that included additional material indices,
+        the phases assigned to those additional material indices would be specified as
+        additional list elements in `phase_labels`.
+    homog_label : str, optional
+        The homogenization scheme label to use for all materials in the volume element. 
+
+    Returns
+    -------
+    volume_element : dict
+
+    """
+
+    geom_dat = read_geom(geom_path)
+    volume_element = {
+        'orientations': {
+            'type': 'euler',
+            'euler_angles': geom_dat['orientations']['euler_angles'],
+        },
+        'element_material_idx': geom_dat['element_material_idx'],
+        'grid_size': geom_dat['grid_size'],
+        'phase_labels': phase_labels,
+        'homog_label': homog_label,
+    }
+    volume_element = validate_volume_element(volume_element)
+    return volume_element
